import {
  stack,
  schemeTableau10,
  extent,
  scaleLinear,
  area,
} from 'd3';
import { areaLabel } from 'd3-area-label';
import { blur } from 'array-blur';

const blurRadius = 15;

export const stacked = (
  selection,
  {
    data,
    height,
    width,
    xValue,
    yValue,
    totalValue,
    mediumKeys,
  }
) => {
  const xScale = d3
    .scaleTime()
    .domain(extent(data, xValue))
    .range([40, width - 40]);

  const yScale = d3
    .scaleLinear()
    .domain([-100, 100])
    .range([height, 0]);

  const xAxis = d3
    .axisBottom(xScale)
    .tickSizeOuter(0);

  const yAxis = d3.axisLeft(yScale);

  const chartData = d3
    .stack()
    .offset(d3.stackOffsetWiggle)
    .keys(mediumKeys)(data);
  const stackedData = chartData;
  console.log(stackedData);

  selection
    .append('text')
    .attr('x', width / 2 - 160)
    .attr('y', 50)
    .style('font-size', 25)
    .style('font-family', 'sans-serif')
    .style('font-color', 'white')
    .attr('font-weight', 700)
    .text('Most Used Medium Over Time');
  
  selection
    .selectAll('g')
    .data(mediumKeys)
    .join('g')
    .attr(
      'transform',
      (d, i) => `translate(${i * 100},70)`
    )
    .call((g) =>
      g
        .append('rect')
        .attr('width', 20)
        .attr('height', 20)
        .style(
          'fill',
          (d, i) => schemeTableau10[i]
        )
    )
    .call((g) =>
      g
        .append('text')
        .attr('y', 10)
        .attr('x', 25)
        .attr('dy', '0.25rem')
        .style('font-size', 10)
        .style('font-family', 'sans-serif')
        .text(
          (d) => d[0] + d.slice(1).toLowerCase()
        )
    );
  var areaGenerator = area()
    .x(function (d) {
      return xScale(d.data.date);
    })
    .y0(function (d) {
      return yScale(d[0]);
    })
    .y1(function (d) {
      return yScale(d[1] || d[0]);
    });

  selection
    .append('g')
    .attr(
      'transform',
      `translate(0,${height - 20})`
    )
    .call(xAxis);

 selection
  .selectAll('mylayers')
  .data(stackedData)
  .enter()
  .append('path')
  .attr('class', 'area')
  .style('fill', (d, i) => schemeTableau10[i])
  .style('opacity', 0.5)
  .attr(
    'd',
    d3
      .area()
      .x(function (d, i) {
        return xScale(d.data.date);
      })
      .y0(function (d) {
        return yScale(d[0]);
      })
      .y1(function (d) {
        return yScale(d[1] || d[0]);
      })
      .curve(d3.curveCardinal)
  )
  .on('mouseover', function (event, d) {
    d3.select(this)
      .style('opacity', 1)
      .style('stroke', 'black')
      .style('stroke-width', '2px');
  })
  .on('mouseout', function (event, d) {
    d3.select(this)
      .style('opacity', 0.5)
      .style('stroke', 'none');
  })
  .append('title')
  .text((d) => d.key);



  const labels = selection
    .append('g')
    .selectAll('text')
    .data(stackedData);

  labels
    .enter()
    .append('text')
    .merge(labels)
    .style('font-family', 'sans-serif')
    .text((d) => d.key)
    .attr('transform', areaLabel(areaGenerator));
};
