import {
  select,
  schemeCategory10,
  csvParse,
  min,
  max,
  extent,
  group,
  flatGroup,
  rollup,
  flatRollup,
  sum,
  bin,
  timeFormat,
} from 'd3';

import { stacked } from './stacked';

export const viz = (
  container,
  { state, setState }
) => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
  const { data, mediumKeys } = state;

  if (data && data !== 'LOADING') {
    svg.call(stacked, {
      data,
      width,
      height,
      xValue: (d) => d.date,
      yValue: (d) => d.medium,
      totalValue: (d) => d.total,
      mediumKeys,
    });
  }

  if (data === undefined) {
    setState((state) => ({
      ...state,
      data: 'LOADING',
    }));

    fetch('data.csv')
      .then((response) => response.text())
      .then((csvString) => {
        const data = csvParse(csvString);
        var parseTime = d3.timeParse('%Y');
        for (const d of data) {
          d.Date = parseTime(
            d.Date.split('-')[0]
          );
        }
        const mediums = flatGroup(
          data,
          (d) => d.Medium.split(' ')[0]
        );
        const keys_ = [];
        for (var i = 0; i < mediums.length; i++) {
          keys_.push(mediums[i][0]);
        }

        const mediumList = [
          'Lithograph',
          'Letterpress',
          'Offset',
          'Poster',
          'Silkscreen',
          'Ink',
          'Glass',
          'Photolithograph',
          'Graphite',
          'Plastic',
        ];
        setState((state) => ({
          ...state,
          mediumKeys: [
            'Lithograph',
            'Letterpress',
            'Offset',
            'Poster',
            'Silkscreen',
            'Ink',
            'Glass',
            'Photolithograph',
            'Graphite',
            'Plastic',
          ],
        }));

        const group = flatGroup(
          data,
          (d) => d.Date
        ).map(([key, value]) => ({
          date: key,
          medium: value
            .map((v) => {
              let medium = v.Medium.split(' ')[0];
              if (mediumList.includes(medium)) {
                return medium;
              }
            })
            .reduce(
              (a, c) => (
                (a[c] = (a[c] || 0) + 1), a
              ),
              {}
            ),
        }));

        let keys = Object.keys(group);
        keys.map((key) => {
          if (
            group[key].hasOwnProperty('medium')
          ) {
            Object.assign(
              group[key],
              group[key]['medium']
            );
            delete group[key]['medium'];
          }
        });
        const finalGroup = group.map((x) => {
          let total = 0;
          Object.keys(x).forEach((key) => {
            if (
              key !== 'date' &&
              key !== 'undefined'
            ) {
              total += x[key];
            }
          });
          return { ...x, total };
        });

        const summary = finalGroup
          .slice(finalGroup)
          .sort((a, b) =>
            d3.ascending(a.date, b.date)
          );

        setState((state) => ({
          ...state,
          data: summary.slice(3, 50),
        }));
      });
  }
};
